<?php

namespace Zeedhi\Translate\Controller;

use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\Session\Session;
use Doctrine\ORM\EntityManager;
use Zeedhi\DevOps\Model\Translator\Entities\ZhWords;
use Zeedhi\DevOps\Model\Translator\Entities\ZhWordsRepository;
use Zeedhi\DevOps\Model\Translator\Entities\ZhRepositoryLanguage;
use Zeedhi\DevOps\Model\Translator\Entities\ZhRepository;
use Zeedhi\Translate\Service\Queue\Producer;
use Zeedhi\Framework\Controller\Crud;

class Words extends Crud {

    protected $dataSourceName = 'words';

    /** @var Session $session */
    protected $session;

    /** @var EntityManager $entityManager */
    protected $entityManager;

    /** @var Producer $wordsProducer */
    protected $wordsProducer;

    /** @var ZhWords $zhWordsRepository */
    protected $zhWordsRepository;

    /** @var ZhRepositoryLanguage $zhRepositoryLanguageRepository */
    protected $zhRepositoryLanguageRepository;

    /** @var ZhRepository $repositoryZhRepository */
    protected $repositoryZhRepository;

    /** @var ZhWordsRepository $zhWordsRepositoryEntity */
    protected $zhWordsRepositoryRepository;

    public function __construct(Manager $manager, Session $session, EntityManager $entityManager, Producer $wordsProducer){
        parent::__construct($manager);

        $this->session = $session;
        $this->entityManager = $entityManager;
        $this->wordsProducer = $wordsProducer;
        $this->zhWordsRepository = $entityManager->getRepository(ZhWords::class);
        $this->repositoryZhRepository = $entityManager->getRepository(ZhRepository::class);
        $this->zhRepositoryLanguageRepository = $entityManager->getRepository(ZhRepositoryLanguage::class);
        $this->zhWordsRepositoryRepository = $entityManager->getRepository(ZhWordsRepository::class);
    }

    protected function setUserSession($userData){
        $this->session->set('USER_ID', $userData['userId']);
        $this->session->set('ORGANIZATION_ID', $userData['organizationId']);
    }

    protected function getWord($wordValue, $birt){
        $persist = false;
        $word = $this->zhWordsRepository->findOneBy(array('value' => $wordValue));
        if (!(isset($word))){
            $word = new ZhWords();
            $word->setValue($wordValue);
            $persist = true;
        }
        if ($birt){
            $word->setBirt('Y');
            $persist = true;
        }
        if ($persist){
            $this->entityManager->persist($word);
        }
        return $word;
    }

    protected function insertWordInRepositoryWords($wordsToSave, $repository, $birt = null){
        foreach ($wordsToSave as $wordValue){
            $word = $this->getWord($wordValue, $birt);
            $wordId = $word->getId();
            $repositoryId = $repository->getId();
            $filter = array(
                'words' => $wordId,
                'repository' => $repositoryId
            );
            $wordInWordsRepository = $this->zhWordsRepositoryRepository->findOneBy($filter);
            if (!(isset($wordInWordsRepository))){
                $wordInWordsRepository = new ZhWordsRepository();
                $wordInWordsRepository->setWords($word);
                $wordInWordsRepository->setRepository($repository);
                $this->entityManager->persist($wordInWordsRepository);
            }
        }
    }

    protected function getRepository($repositoryId){
        $repository = $this->repositoryZhRepository->findOneBy(array('id' => $repositoryId));
        if ($repository == null){ //|| $repository->getStatus() != 'A'){
            throw new \Exception("Nonexistent or inactive repository");
        }
        return $repository;
    }

    public function saveWords(Request\DataSet $request, Response $response){
        $row = $request->getDataSet()->getRows()[0];
        $userData = $row['info'];
        $wordsToSave = $row['json'];
        $birtWordsToSave = $row['birt'];
        $repositoryId = $userData['repositoryId'];
        $this->setUserSession($userData);
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            $repository = $this->getRepository($repositoryId);
            $this->insertWordInRepositoryWords($wordsToSave, $repository);
            $this->insertWordInRepositoryWords($birtWordsToSave, $repository, true);
            $this->entityManager->flush();
            $connection->commit();
            $this->sendWordsToTranslate($repository);
            $response->addMessage(new Response\Message('WORDS_SAVED'));
        } catch (\Exception $e){
            $connection->rollBack();
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    protected function getRepositoryLanguages($repository){
        $filter = array( "repository" => $repository);
        $this->entityManager->beginTransaction();
        return $this->entityManager->getRepository(ZhRepositoryLanguage::class)->findBy($filter);
    }

    protected function sendWordsToTranslate($repository){
        $repositoryId = $repository->getId();
        $repositoryLanguageFrom = $repository->getDefaultLanguage();
        foreach ($this->getRepositoryLanguages($repository) as $language) {
            $dataToSend = array(
                "repositoryId" => $repositoryId,
                "languageFrom" => $repositoryLanguageFrom,
                "languageTo" => $language->getLanguage()->getId()
            );
            $this->wordsProducer->send(json_encode($dataToSend));
        }
    }
}